@@ -1,6 +1,6 @@
 /*
 
-Copyright (c) 2003-2016, Arvid Norberg
+Copyright (c) 2003-2018, Arvid Norberg
 All rights reserved.
 
 Redistribution and use in source and binary forms, with or without
@@ -94,7 +94,7 @@ namespace libtorrent
 #ifndef TORRENT_NO_DEPRECATE
 	// thrown by bdecode() if the provided bencoded buffer does not contain
 	// valid encoding.
-	struct invalid_encoding: std::exception
+	struct TORRENT_DEPRECATED invalid_encoding : std::exception
 	{
 		// hidden
 		virtual const char* what() const TORRENT_EXCEPTION_THROW_SPECIFIER
@@ -210,10 +210,17 @@ namespace libtorrent
 				write_char(out, 'e');
 				ret += 2;
 				break;
+			case entry::preformatted_t:
+				std::copy(e.preformatted().begin(), e.preformatted().end(), out);
+				ret += e.preformatted().size();
+				break;
 			case entry::undefined_t:
-				// trying to encode a structure with uninitialized values!
-//				TORRENT_ASSERT_VAL(false, e.type());
-				// do nothing
+
+				// empty string
+				write_char(out, '0');
+				write_char(out, ':');
+
+				ret += 2;
 				break;
 			}
 			return ret;
