@@ -1,6 +1,6 @@
 /*
 
-Copyright (c) 2005-2016, Arvid Norberg
+Copyright (c) 2005-2018, Arvid Norberg
 All rights reserved.
 
 Redistribution and use in source and binary forms, with or without
@@ -194,13 +194,6 @@ POSSIBILITY OF SUCH DAMAGE.
 #if MAC_OS_X_VERSION_MIN_REQUIRED >= 1050
 #define TORRENT_USE_EXECINFO 1
 #endif
-
-#else // __APPLE__
-// FreeBSD has a reasonable iconv signature
-// unless we're on glibc
-#ifndef __GLIBC__
-# define TORRENT_ICONV_ARG (const char**)
-#endif
 #endif // __APPLE__
 
 #define TORRENT_HAVE_MMAP 1
@@ -344,7 +337,6 @@ POSSIBILITY OF SUCH DAMAGE.
 #define TORRENT_USE_IFCONF 1
 #define TORRENT_USE_SYSCTL 1
 #define TORRENT_USE_IPV6 0
-#define TORRENT_ICONV_ARG (const char**)
 #define TORRENT_USE_WRITEV 0
 #define TORRENT_USE_READV 0
 
@@ -456,10 +448,6 @@ int snprintf(char* buf, int len, char const* fmt, ...)
 #define TORRENT_OVERRIDE override
 #endif
 
-#ifndef TORRENT_ICONV_ARG
-#define TORRENT_ICONV_ARG (char**)
-#endif
-
 #if defined __GNUC__ || defined __clang__
 #define TORRENT_FORMAT(fmt, ellipsis) __attribute__((__format__(__printf__, fmt, ellipsis)))
 #else
@@ -728,6 +716,9 @@ int snprintf(char* buf, int len, char const* fmt, ...)
 
 #endif // TORRENT_HAS_SSE
 
+namespace libtorrent {}
+
+namespace lt = libtorrent;
 
 #endif // TORRENT_CONFIG_HPP_INCLUDED
 
